﻿using LawFirmContracts.BindingModels;
using LawFirmContracts.SearchModels;
using LawFirmContracts.StorageContracts;
using LawFirmContracts.ViewModels;
using LawFirmFileImplement.Models;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace LawFirmFileImplement.Implements
{
    public class ShopStorage : IShopStorage
    {
        private readonly DataFileSingleton source;
        public ShopStorage()
        {
            source = DataFileSingleton.GetInstance();
        }
        public ShopViewModel? GetElement(ShopSearchModel model)
        {
            if (!model.Id.HasValue)
            {
                return null;
            }
            return source.Shops.FirstOrDefault(x => model.Id.HasValue && x.Id == model.Id)?.GetViewModel;
        }

        public List<ShopViewModel> GetFilteredList(ShopSearchModel model)
        {
            if (string.IsNullOrEmpty(model.Name))
            {
                return new();
            }
            return source.Shops
                .Select(x => x.GetViewModel)
                .Where(x => x.Name.Contains(model.Name ?? string.Empty))
                .ToList();
        }

        public List<ShopViewModel> GetFullList()
        {
            return source.Shops.Select(shop => shop.GetViewModel).ToList();
        }

        public ShopViewModel? Insert(ShopBindingModel model)
        {
            model.Id = source.Shops.Count > 0 ? source.Shops.Max(x => x.Id) + 1 : 1;
            var newShop = Shop.Create(model);
            if (newShop == null)
            {
                return null;
            }
            source.Shops.Add(newShop);
            source.SaveShops();
            return newShop.GetViewModel;
        }

        public ShopViewModel? Update(ShopBindingModel model)
        {
            var shop = source.Shops.FirstOrDefault(x => x.Id == model.Id);
            if (shop == null)
            {
                return null;
            }
            shop.Update(model);
            source.SaveShops();
            return shop.GetViewModel;
        }
        public ShopViewModel? Delete(ShopBindingModel model)
        {
            var shop = source.Shops.FirstOrDefault(x => x.Id == model.Id);
            if (shop == null)
            {
                return null;
            }
            source.Shops.Remove(shop);
            source.SaveShops();
            return shop.GetViewModel;
        }

        public bool SellDocument(DocumentSearchModel model, int count)
        {
            var document = source.Documents.FirstOrDefault(x => model.Id.HasValue && x.Id == model.Id);

            var countStore = count;

            if (document == null)
            {
                return false;
            }

            foreach (var shop in source.Shops)
            {
                foreach (var doc in shop.ShopDocuments)
                {
                    if (doc.Value.Item1.Id == document.Id)
                    {
                        if (count > 0)
                        {
                            count = count - doc.Value.Item2;
                        }
                    }
                    if (count < 1)
                    {
                        break;
                    }
                }
            }

            if (count > 0)
            {
                return false;
            }

            count = countStore;

            for (int i = 0; i < source.Shops.Count; i++)
            {
                var shop = source.Shops[i];
                var documents = shop.ShopDocuments;

                for (int j = 0; j < documents.Count; j++)
                {
                    if (documents[j].Item1.Id == document.Id)
                    {
                        while (documents[j].Item2 > 0 || count > 0)
                        {
                            int tempItem2 = documents[j].Item2;
                            tempItem2--;
                            count--;
                            documents[j] = (documents[j].Item1, tempItem2);
                        }
                    }

                    if (count < 1)
                    {
                        shop.Update(new ShopBindingModel
                        {
                            Id = shop.Id,
                            Name = shop.Name,
                            Adress = shop.Adress,
                            OpeningDate = shop.OpeningDate,
                            MaxCountDocuments = shop.MaxCountDocuments,
                            ShopDocuments= documents
                        });
                        break;
                    }
                }
                shop.Update(new ShopBindingModel
                {
                    Id = shop.Id,
                    Name = shop.Name,
                    Adress = shop.Adress,
                    OpeningDate = shop.OpeningDate,
                    MaxCountDocuments = shop.MaxCountDocuments,
                    ShopDocuments = documents
                });
            }

            if (count > 0)
            {
                return false;
            }
            return true;
        }
    }
}
